package me.paradoxpixel.themepark.config;

import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.Plugin;
import java.io.File;

public class YamlConfig {

    private String name;
    private File file;
    private FileConfiguration config;
    private Plugin plugin;

    public YamlConfig(String name, Plugin plugin) {
        this.name = name.endsWith(".yml") ? name : name + ".yml";
        this.plugin = plugin;

        load();
    }

    public void load() {
        if(!plugin.getDataFolder().exists())
            plugin.getDataFolder().mkdir();

        file = new File(plugin.getDataFolder(), name);
        if(!file.exists()) {
            if(plugin.getResource(name) == null) {
                try {
                    if(!file.createNewFile())
                        System.out.print("Couldn't create config file: " + name);
                } catch(Exception e) {
                    e.printStackTrace();
                    return;
                }
            } else {
                plugin.saveResource(name, true);
            }
        }

        config = YamlConfiguration.loadConfiguration(file);
    }

    public FileConfiguration getConfig() {
        return config;
    }

    public void reload() {
        if(file == null) {
            load();
            return;
        }

        config = YamlConfiguration.loadConfiguration(file);
    }

    public void save() {
        try {
            config.save(file);
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    public YamlConfig copy(String name) {
        name = name.endsWith(".yml") ? name : name + ".yml";

        File file = new File(plugin.getDataFolder(), name);
        try {
            config.save(file);
        } catch(Exception e) {
            e.printStackTrace();
            return null;
        }

        return new YamlConfig(name, plugin);
    }

    public void delete() {
        try {
            if(!file.delete())
                System.out.print("Couldn't delete config file: " + name);
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
}
